#include "stm32f10x.h"

#ifndef __TPS929120_H
#define __TPS929120_H

#define WRITE_MASK            0x80
#define READ_MASK             0x7F
#define ENABLE_TPS_OUT        0x01
#define DISABLE_TPS_OUT       0x00

//Definicje wyjsc dla WriteOutTPS(uint8_t adress, uint16_t outputs); @arg outputs - naley podawac jako suma np.
//TPSOUT0 | TPSOUT3 | TPSOUT7 - jezeli chcemy miec zalaczone wyjscia 0, 3 oraz 7
#define TPSOUT0               (1<<0)
#define TPSOUT1               (1<<1)
#define TPSOUT2               (1<<2)
#define TPSOUT3               (1<<3)
#define TPSOUT4               (1<<4)
#define TPSOUT5               (1<<5)
#define TPSOUT6               (1<<6)
#define TPSOUT7               (1<<7)
#define TPSOUT8               (1<<8)
#define TPSOUT9               (1<<9)
#define TPSOUT10              (1<<10)
#define TPSOUT11              (1<<11)

//Definicje dla funkcji konfiguracji wartosic wyjsc oraz dla funkcji, sluzacych do programowania EEPROM
#define ICONFBASEADDR         0x00
#define PWMBASEADDR           0x20
#define CONF_EEPGATE          0x65
#define CONF_EEPMODE          0x63
#define CONF_STAYINEEP        0x62
#define CONF_EEPPROG          0x64
#define EEPIBASE              0x80
#define EEPPBASE              0xA0

//Definicje do okreslnia z ktorego kanalu chcemy mierzyc wartosc ADC
#define ADCREF                0x00
#define ADCSUPPLY             0x01
#define ADCVLDO               0x02
#define ADCTEMPSNS            0x03
#define ADCIREF               0x04
#define ADCMAXOUT             0x05
#define ADCOUT0               0x10
#define ADCOUT1               0x11
#define ADCOUT2               0x12
#define ADCOUT3               0x13
#define ADCOUT4               0x14
#define ADCOUT5               0x15
#define ADCOUT6               0x16
#define ADCOUT7               0x17
#define ADCOUT8               0x18
#define ADCOUT9               0x19
#define ADCOUT10              0x1A
#define ADCOUT11              0x1B

//Definicje dla okreslenia przy wywolaniu funkcji konfiguracyjnej ktorego kanalu nalezy uzyc
#define CONF_DISABLE_OUTPUTS 0
#define	CONF_ENCH0 1
#define	CONF_ENCH1 2
#define	CONF_ENCH2 4
#define	CONF_ENCH3 8
#define	CONF_ENCH4 16
#define	CONF_ENCH5 32
#define	CONF_ENCH6 64
#define	CONF_ENCH7 128
#define	CONF_ENCH8 256
#define	CONF_ENCH9 512
#define	CONF_ENCH10 1024
#define	CONF_ENCH11 2048
#define	CONF_ENCH12 4096

//Definicje dla okreslenia przy wywolaniu funkcji konfiguracyjnego, na ktorym kanale ma byc zalaczona,
//lub wylaczona diagnostyka
#define	CONF_DISABLE_ALL_DIAG 0
#define	CONF_DIAGCH0 1
#define	CONF_DIAGCH1 2
#define	CONF_DIAGCH2 4
#define	CONF_DIAGCH3 8
#define	CONF_DIAGCH4 16
#define	CONF_DIAGCH5 32
#define	CONF_DIAGCH6 64
#define	CONF_DIAGCH7 128
#define	CONF_DIAGCH8 256
#define	CONF_DIAGCH9 512
#define	CONF_DIAGCH10 1024
#define	CONF_DIAGCH11 2048
#define	CONF_DIAGCH12 4096

//Definicje dla wykasowania flag FAUL lub POR
#define POR_CLR   (1<<0)
#define FAULT_CLR (1<<1)
#define ERR_PULL_DOWN (1<<3)

typedef struct{
	uint8_t ICONF;
	uint8_t PWM;
	uint8_t OUT;
} ConNormalMode;

typedef struct{
	uint8_t EEPI;
	uint8_t EEPP;
	uint8_t OUT;
} ConFSMode;

typedef struct{
	uint8_t FLAG0;
	uint8_t FLAG1;
	uint8_t FLAG2;
	uint8_t FLAG3;
	uint8_t FLAG4;
	uint8_t FLAG5;
	uint8_t FLAG7;
	uint8_t FLAG8;
	uint8_t FLAG11;
	uint8_t FLAG12;
	uint8_t FLAG13;
	uint8_t FLAG14;
} FlagStruct;

typedef enum{
	CONF_EN0=0x50,
	CONF_EN1,
	CONF_DIAGEN0=0x54,
	CONF_DIAGEN1,
	CONF_MISC0,
	CONF_MISC1,
	CONF_MISC2,
	CONF_MISC3,
	CONF_MISC4,
	CONF_MISC5,
	CLR=0x60,
	CONF_LOCK,
	CONF_MISC6,
	CONF_MISC7,
	CONF_MISC8,
	CONF_MISC9
} ConfigRegisters;

typedef enum{
	CONF_REFRANGE64=0,
	CONF_REFRANGE128,
	CONF_REFRANGE256,
	CONF_REFRANGE512
} ConfRANGE;

typedef enum{
	CONF_EXPEN_DISABLE=0,
	CONF_EXPEN_ENABLE
} ConfEXPEN;

typedef enum{
	CONF_LDO_5_0=0,
	CONF_LDO_4_4
} ConfLDO;

typedef enum{
	CONF_AUTOSS_DISABLE=0,
	CONF_AUTOSS_ENABLE
} ConfAUTOSS;

typedef enum{
	CONF_PWMF_200=0,
	CONF_PWMF_250,
	CONF_PWMF_300,
	CONF_PWMF_350,
	CONF_PWMF_400,
	CONF_PWMF_500,
	CONF_PWMF_600,
	CONF_PWMF_800,
	CONF_PWMF_1000,
	CONF_PWMF_1200,
	CONF_PWMF_2k,
	CONF_PWMF_4k,
	CONF_PWMF_5_9k,
	CONF_PWMF_7_8k,
	CONF_PWMF_9_6k,
	CONF_PWMF_20_8k
} CONF_PWMF;

typedef enum{
	CONF_FLTIMEOUT_1MS=0,
	CONF_FLTIMEOUT_125US,
	CONF_FLTIMEOUT_250US,
	CONF_FLTIMEOUT_500US,
	CONF_FLTIMEOUT_1_25MS,
	CONF_FLTIMEOUT_2_5MS,
	CONF_FLTIMEOUT_5MS,
	CONF_FLTIMEOUT_10MS
}CONF_TIMEOUT;

typedef enum{
	CONF_ADCLOWSUPTH_5V=0,
	CONF_ADCLOWSUPTH_6V,
	CONF_ADCLOWSUPTH_7V,
	CONF_ADCLOWSUPTH_8V,
	CONF_ADCLOWSUPTH_9V,
	CONF_ADCLOWSUPTH_10v,
	CONF_ADCLOWSUPTH_11V,
	CONF_ADCLOWSUPTH_12V,
	CONF_ADCLOWSUPTH_13V,
	CONF_ADCLOWSUPTH_14V,
	CONF_ADCLOWSUPTH_15V,
	CONF_ADCLOWSUPTH_16V,
	CONF_ADCLOWSUPTH_17V,
	CONF_ADCLOWSUPTH_18V,
	CONF_ADCLOWSUPTH_19V,
	CONF_ADCLOWSUPTH_20V
}CONF_ADCSUPTH;

typedef enum{
	CONF_ODPW_100us=0,
	CONF_ODPW_20us,
	CONF_ODPW_30us,
	CONF_ODPW_50us,
	CONF_ODPW_80us,
	CONF_ODPW_150us,
	CONF_ODPW_200us,
	CONF_ODPW_300us,
	CONF_ODPW_500us,
	CONF_ODPW_800us,
	CONF_ODPW_1ms,
	CONF_ODPW_1_2ms,
	CONF_ODPW_1_5ms,
	CONF_ODPW_2ms,
	CONF_ODPW_3ms,
	CONF_ODPW_5ms
} TPSODPW;

typedef enum{
	CONF_WDTIMER_DISABLE=0,
	CONF_WDTIME_200us,
	CONF_WDTIME_500us,
	CONF_WDTIME_1ms,
	CONF_WDTIME_2ms,
	CONF_WDTIME_5ms,
	CONF_WDTIME_10ms,
	CONF_WDTIME_20ms,
	CONF_WDTIME_50ms,
	CONF_WDTIME_100ms,
	CONF_WDTIME_200ms,
	CONF_WDTIME_500ms,
	CONF_WDTIME_DIRECT_ENTER_TO_FS_MODE=0x0f

} ConWDT;

typedef struct{
	uint16_t CONF_ENCHx;           //Zalaczenie okreslonych wyjsc
	uint16_t CONF_DIAGCHx;         //zalaczenie lub wylaczenie diagnostyki
	ConfRANGE CONF_REFRANGEx;      //Zalaczenie okreslonego zakresu
	ConfEXPEN CONF_EXPx;           //eksponencalne lub liniowe sterowanie ledami
	ConfLDO   CONF_LDOx;           //wybranie napiecia na wyjsciu LDO
	ConfAUTOSS CONF_AUTOSSx;       //Automatyczna diagnostyka
	CONF_PWMF CONF_PWMFREQ;        //Ustawienie czestotliwosci PWM
	CONF_TIMEOUT CONF_FLTIMEOUT;   //Ustawienie timeout - czasu pomiedzy ramkami danych
	CONF_ADCSUPTH CONF_ADCLOWSUPTH;//Ustawienie progru napiecia od ktorego ma zostac dopiero aktywowana diagnostyka
	uint8_t CONF_ODIOUT;           //Wartosc domyslna 0x00
	TPSODPW CONF_ODPW;             //wartosc domyslna 100us
	ConWDT CONF_WDTIMER;          //wartosc domyslna 0 - wylaczony wathdog, parametr CONF_WDTIME_DIRECT_ENTER_TO_FS_MODE - powoduje przejscie odrazu do stanu FS_MODE
} ConfStruct;

typedef enum{
	CONF_EXTCLK_DISABLE=0,
	CONF_EXTCLK_ENABLE
} TPSEXTCLK;

typedef enum{
	CONF_SHAREPWM_DISABLE=0,
	CONF_SHAREPWM_ENABLE
} TPSSHAREPWM;

typedef enum{
	CONF_READSHADOW_DISABLE=0,
	CONF_READSHADOW_ENABLE
} TPSREADSHADOW;

typedef struct{
	TPSEXTCLK CONF_EXTCLK;
	TPSSHAREPWM CONF_SHAREPWM;
	TPSREADSHADOW CONF_READSHADOW;
} Misc_7_Struct;

typedef enum{
	CONF_MASKTSD_DISABLE=0,
	CONF_MASKTSD_ENABLE
} TPSMASKTSD;

typedef enum{
	CONF_MASKOPEN_DISABLE=0,
	CONF_MASKOPEN_ENABLE
} TPSMASKOPEN;

typedef enum{
	CONF_MASKSHORT_DISABLE=0,
	CONF_MASKSHORT_ENABLE
}TPSMASKSHORT;

typedef enum{
	CONF_MASKCRC_DISABLE=0,
	CONF_MASKCRC_ENABLE
} TPSMASKCRC;

typedef enum{
	CONF_MASKREF_DISABLE=0,
	CONF_MASKREF_ENABLE
}TPSMASKREF;

typedef struct{
	TPSMASKTSD CONF_MASKTSD;
	TPSMASKOPEN CONF_MASKOPEN;
	TPSMASKSHORT CONF_MASKSHORT;
	TPSMASKCRC CONF_MASKCRC;
	TPSMASKREF CONF_MASKREF;
} ERR_Structure_Mask;

uint8_t Calculate_CRC8(uint8_t *byteVal, uint8_t numbersofbyte, uint8_t generator, uint8_t initvalue, uint8_t checkcrc);
uint8_t TPS929120(uint8_t DEV_ADDR, uint8_t REG_ADDR, uint8_t DATA_N, volatile uint8_t *wskRX,volatile uint8_t *wskbuf);
uint8_t reflectforcrc(uint8_t *inputtable, uint8_t *outputtable, uint8_t amoundbytes);
void SendBytesViaUART( uint8_t *wsktab, uint8_t amoundbytes);
void Write_data_TPS(uint8_t adress, uint8_t regadress, uint8_t data);
uint8_t Read_data_TPS(uint8_t adress, uint8_t regadress);
void Control_Output_TPS(uint8_t adress, uint8_t outnumber, uint8_t enabledisable);
void Set_Bit_TPS(uint8_t adress, uint8_t regadress, uint8_t bitnumber);
void Clear_Bit_TPS(uint8_t adress, uint8_t regadress, uint8_t bitnumber);
void Unlock_TPS(uint8_t adress);
void TPS_Setup(uint8_t adress, ConNormalMode *param);
void Lock_TPS(uint8_t adress);
void WriteOutTPS(uint8_t adress, uint16_t outputs);
void delaymsTPS(uint32_t ms);
void Write_EEPROM_TPS(uint8_t adress);
void Unlock_EEPROM_TPS(uint8_t adress);
void EEPROM_FAIL_STATE(uint8_t adress, uint16_t FS0, uint16_t FS1);
void TPS_EEP_Setup(uint8_t adress, ConFSMode *param);
void TPS_Force_FS(uint8_t adress);
void Lock_EEPROM_TPS(uint8_t adress);
void Default_EEPROM_TPS_BY_REF(uint8_t adress);
void TPS_Return_FS(uint8_t adress);
void Default_EEPROM_TPS_BY_ADDR(uint8_t adress);
void Write_CRC_EEPROM(uint8_t adress);
void Read_ALL_EEPROM(uint8_t adress,uint8_t *data);
uint8_t getADC(uint8_t adress, uint8_t channel);
long int calcvalueADC(uint8_t adress, uint8_t channel);
FlagStruct ReadFlags(uint8_t adress);
uint8_t ReadConfRegister(uint8_t adress, ConfigRegisters confregister);
uint8_t WDT_Reset_TPS(uint8_t adress);
void Write_Conf_TPS(uint8_t adress, ConfStruct *wsk);
void Write_Misc_7_TPS(uint8_t adress, Misc_7_Struct *wsk);
void ERR_Conf_Mask(uint8_t adress, ERR_Structure_Mask *wsk);
void CLR_write(uint8_t adress, uint8_t value);
void PullDownERR(uint8_t adress);
void ProgramEEPforFS(uint8_t adress);
void Read_ALL_EEPROM_CRC(uint8_t adress,uint8_t *data);

#endif

